// Copyright 2008-2009  Segher Boessenkool  <segher@kernel.crashing.org>
// Copyright 2003-2004  Felix Domke <tmbinc@elitedvb.net>
// This code is licensed to you under the terms of the GNU GPL, version 2;
// see file COPYING or http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt

#include "loader.h"

extern u8 console_font_10x16x4[];

#define FONT_XSIZE  10
#define FONT_YSIZE  16
#define FONT_XGAP   0
#define FONT_YGAP   2

static struct {
	u32 xres, yres, stride;

	u32 cursor_x, cursor_y;

	u32 border_left, border_right, border_top, border_bottom;
} fb;

static void fb_write(u32 offset, u32 x)
{
//	write32(0x00f00000 + offset, x);
	u32 *p = (u32 *)(0x80f00000 + offset);
	*p = x;
	sync_after_write(p, 4);
}

static void fb_clear_lines(u32 top, u32 lines)
{
	u32 x, y;
	u32 offset;

	offset = fb.stride * top;

	for (y = 0; y < lines; y++) {
		for (x = 0; x < fb.xres/2; x++)
			fb_write(offset + 4*x, 0x00800080);

		offset += fb.stride;
	}
}

void dbvideocollor(u32 collor)
{
	u32 x, y;
	u32 offset;

	offset = fb.stride;

	for (y = 0; y < fb.yres; y++) {
		for (x = 0; x < fb.xres/2; x++)
			fb_write(offset + 4*x, collor);

		offset += fb.stride;
	}
}

static void fb_init(u32 xres, u32 yres, u32 stride)
{
	fb.xres = xres;
	fb.yres = yres;
	fb.stride = stride;

	fb.border_left = 30;
	fb.border_top = 30;
	fb.border_right = fb.xres - 30;
	fb.border_bottom = fb.yres - 30;

	fb.cursor_x = fb.border_left;
	fb.cursor_y = fb.border_top;

	fb_clear_lines(0, fb.yres);
}

void video_init(void)
{
	// read VTR register to determine linecount and mode
	u32 vtr = read16(0x0c002000);
	u32 lines = vtr >> 4;

	if ((vtr & 0x0f) > 10) {	// progressive
		// set framebuffer position
		write32(0x0c00201c, 0x00f00000);
		write32(0x0c002024, 0x00f00000);
	} else {			//interlaced
		lines *= 2;

		u32 vto = read32(0x0c00200c);
		u32 vte = read32(0x0c002010);

		// set framebuffer position
		// try to figure out the interlacing order
		if ((vto & 0x03ff) < (vte & 0x03ff)) {
			write32(0x0c00201c, 0x00f00000);
			write32(0x0c002024, 0x00f00000 + 2*640);
		} else {
			write32(0x0c00201c, 0x00f00000 + 2*640);
			write32(0x0c002024, 0x00f00000);
		}
	}

	fb_init(640, lines, 2*640);
}
